/*
 * Copyright (c) Constantin Mircea Moisei 2005 for AjaxAnywhere
 * 
 * $Header: /cvsroot/ajaxanywhere/ajaxAnywhere/src/java/org/ajaxanywhere/LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ 
 * $Revision: 1.4 $ 
 * $Date: 2006/02/20 06:41:06 $
 * 
 */
package org.ajaxanywhere;


import java.text.MessageFormat;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;

/**
 *  <b>org.ajaxanywhere.LinkTag</b> <br>
 * 
 * It's for the failover mechanism. It's going to be transparent for the user how it works.
 * 
 * <p><u>Usage</u></p>
 * <code>
 * <aa:link value="/some.url" zones="content,menu"/>Ajax Link<aa:link><br>
 * <aa:link value="/some.url" zones="content,menu" functionName="ajaxanywhere.getAJAXAndClean" />Ajax Link<aa:link><br>
 * <aa:link  value="/some.url" zones="zone1, zone2" functionName="openLinkTop" style="color:red" styleClass="class">Ajax Link</aa:link>
 * </code>
 * 
 * <p>The tag will expand to <br>
 * <a href="/some.url" onclick="ajaxanywhere.getAjax( '/some.url', 'content,menu');return false;">Ajax Link</a><br>
  * <a href="/some.url" onclick="ajaxanywhere.getAJAXAndClean( '/some.url', 'content,menu');return false;">Ajax Link</a><br>
 *  <a href="/some.url" onclick="openLinkTop( '/some.url', 'content,menu');return false;" style="color:red" class="class">Ajax Link</a><br>
 * 
 * </p>
 * 
 * <p>Both getAJAXAndClean and openLinkTop are defined in aa.js</p>
 * 
 * @author Constantin Moisei
 * @version $Revision: 1.4 $
 * @since Feb 18, 2006 10:48:52 AM
 * 
 * 
 */
public class LinkTag extends BodyTagSupport //org.apache.taglibs.standard.tag.el.core.UrlTag
{

    /**
     * Comment for <code>serialVersionUID</code> long
     */
    private static final long serialVersionUID = 1927146406417618943L;

    public static final String LINK_HTML_START = "<a href=\"{0}\" onclick=\"{1}\" {2} >";
    public static final String LINK_HTML_END = "</a>";
    public static final String DEFAULT_FUNCTION_NAME = "ajaxAnywhere.getAJAX";

    //attributes
    String functionName;
    String zones;
    String target;
    String value;
    String styleClass;
    String style;
    

    public LinkTag( )
    {
        super();
        functionName = DEFAULT_FUNCTION_NAME;
    }

    /**
     * @return Returns the function String.
     */
    public String getFunctionName()
    {
        return functionName;
    }

    /**
     * @param function
     *            The function to set. Type String
     */
    public void setFunctionName( String function )
    {
        this.functionName = function;
    }

    private Object buildOnClick()
    {
        StringBuffer strbuff = new StringBuffer( this.getFunctionName() );
        strbuff.append( "('" );
        strbuff.append( buildUrl() );
        strbuff.append( "', '" );
        strbuff.append( this.getZones() );
        strbuff.append( "')" );
        return strbuff.toString();
    }
    /**
     * 
     * @return
     * Object
     */
    private Object buildOtherAttributes()
    {
        StringBuffer strbuff = new StringBuffer( );
        
        if ( target != null && target.trim().length() > 0 )
        {
            strbuff.append( "target=\"" );
            strbuff.append( target );
            strbuff.append( "\"" );
            strbuff.append( " " );            
        }
        
        if ( style != null && style.trim().length() > 0 )
        {
            strbuff.append( "style=\"" );
            strbuff.append( style );
            strbuff.append( "\"" );
            strbuff.append( " " );            
        }
        
        if ( styleClass != null && styleClass.trim().length() > 0 )
        {
            strbuff.append( "class=\"" );
            strbuff.append( styleClass );
            strbuff.append( "\"" );
            strbuff.append( " " );            
        }
       
        return strbuff.toString();
    }

    /**
     * @return Returns the zones String.
     */
    public String getZones()
    {
        return zones;
    }

    /**
     * @param zones
     *            The zones to set. Type String
     */
    public void setZones( String zones )
    {
        this.zones = zones;
    }
    
    /**
     * 
     */
    public int doStartTag() throws JspException
    {
        try
        {
            Object[] arguments = {  buildUrl(), buildOnClick(), buildOtherAttributes() };
            String html = MessageFormat.format( LINK_HTML_START, arguments );
            pageContext.getOut().print( html );
        }
        catch ( Exception ex )
        {
            throw new JspException( ex );
        }
        return EVAL_BODY_BUFFERED;//EVAL_BODY_TAG;
    }
    
    
   

     /**
      * Build url using encodeURL and <code>value</code>
      * @return
      * Object
      */   
    private Object buildUrl()
    {
        HttpServletRequest request = ( HttpServletRequest ) pageContext.getRequest();
        return ( ( HttpServletResponse ) this.pageContext.getResponse() ).encodeURL( request.getContextPath() + "/" + value );
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.taglibs.standard.tag.common.core.UrlSupport#doEndTag()
     * 
     * @return @throws JspException
     * 
     */
    public int doEndTag() throws JspException
 {
        
        try
        {
            BodyContent bc = getBodyContent();
            // get the bodycontent as string
            String body = bc.getString();
            // getJspWriter to output content
            JspWriter out = bc.getEnclosingWriter();
            if ( body != null )
            {
                out.print( body );
                pageContext.getOut().print( LINK_HTML_END );
            }
        }
        catch ( Exception e )
        {
            throw new JspException( e );
        }
        // return SKIP_BODY;
        return EVAL_PAGE;
    }

    
    /**
     * @return Returns the target String.
     */
    public String getTarget()
    {
        return target;
    }

    
    /**
     * @param target The target to set. Type String
     */
    public void setTarget( String target )
    {
        this.target = target;
    }

    
    /**
     * @return Returns the style String.
     */
    public String getStyle()
    {
        return style;
    }

    
    /**
     * @param style The style to set. Type String
     */
    public void setStyle( String style )
    {
        this.style = style;
    }

    
    /**
     * @return Returns the styleClass String.
     */
    public String getStyleClass()
    {
        return styleClass;
    }

    
    /**
     * @param styleClass The styleClass to set. Type String
     */
    public void setStyleClass( String styleClass )
    {
        this.styleClass = styleClass;
    }

    
    /**
     * @return Returns the value String.
     */
    public String getValue()
    {
        return value;
    }

    
    /**
     * @param value The value to set. Type String
     */
    public void setValue( String value )
    {
        this.value = value;
    }

}

// end class LinkTag

/*
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ $Log: LinkTag.java,v $
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ Revision 1.4  2006/02/20 06:41:06  mcmoisei
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ onclick fix
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ Revision 1.3  2006/02/19 00:03:12  mcmoisei
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ Javadoc fixes
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ Revision 1.2  2006/02/18 23:49:28  mcmoisei
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ Javadoc fixes
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ Revision 1.1  2006/02/18 23:40:11  mcmoisei
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ This is the very first release of the tag aa:link
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $ This by default expands to <a href="url" onclick="ajaxanywhere.getAjax( 'url', 'zone1, zone2, ...,zoneN');return false;">Ajax Link</a>
 * $Id: LinkTag.java,v 1.4 2006/02/20 06:41:06 mcmoisei Exp $
 * 
 */