package org.ajaxanywhere.jsf;

import org.ajaxanywhere.AAUtils;

import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.servlet.ServletRequest;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;


public class ZoneUIComponent extends UIComponentBase {
    private static final String COMPONENT_FAMILY = "AjaxAnywhereFamily";

    public ZoneUIComponent() {
        setTransient(false);
    }

    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    public void encodeBegin(FacesContext context)
            throws IOException {
        if (context == null) throw new IllegalArgumentException("context is null");
        if (!isRendered()) return;
        if (getId() == null)
            throw new IllegalArgumentException("name attribute is null");

        if (canSkipRendering(this, context))
            return;

        ResponseWriter writer = context.getResponseWriter();
        writer.write(AAUtils.getZoneStartDelimiter(getId()));

    }

    public boolean getRendersChildren() {
        return true;
    }

    public void encodeChildren(FacesContext context)
            throws IOException {
        if (context == null) throw new IllegalArgumentException("context is null");
        if (canSkipRendering(this, context))
            return;

         renderChildren(context, this);
    }

    public void encodeEnd(FacesContext context)
            throws IOException {
        if (context == null) throw new IllegalArgumentException("context is null");
        if (!isRendered()) return;
        if (canSkipRendering(this, context))
            return;

        ResponseWriter writer = context.getResponseWriter();
        writer.write(AAUtils.getZoneEndDelimiter(getId()));
    }


    private static void renderChildren(FacesContext facesContext, UIComponent component)
            throws IOException {
        if (component.getChildCount() > 0) {
            for (Iterator it = component.getChildren().iterator(); it.hasNext();) {
                UIComponent child = (UIComponent) it.next();
                renderComponent(facesContext, child);
            }
        }
    }


    private static void renderComponent(FacesContext facesContext, UIComponent child)
            throws IOException {

        if (!child.isRendered()) {
            return;
        }

        child.encodeBegin(facesContext);
        if (child.getRendersChildren()) {
            child.encodeChildren(facesContext);
        } else {
            renderChildren(facesContext, child);
        }
        child.encodeEnd(facesContext);
    }

    private static boolean canSkipRendering(UIComponent component, FacesContext context) {
        return noIncludedSubZones(component, context) && notInsideIncludedZone(component, context);
    }
    /**
     * checks if this zones is surrently inside another included zone
     */
    private static boolean notInsideIncludedZone(UIComponent component, FacesContext context) {
        Map requestMap = context.getExternalContext().getRequestMap();
        UIComponent parent = component.getParent();
        while (parent != null && ! (parent instanceof UIViewRoot)) {
            if (parent instanceof ZoneUIComponent && AAUtils.getZonesToRefresh(requestMap).contains(parent.getId()))
                return false;
            parent = parent.getParent();
        }
        return true;
    }

    /**
     * checks if there are any sun-zones to include
     */
    private static boolean noIncludedSubZones(UIComponent component, FacesContext context) {
        Map requestMap = context.getExternalContext().getRequestMap();
        Map requestParameterMap = context.getExternalContext().getRequestParameterMap();
        if (! AAUtils.isAjaxRequest(requestParameterMap) )
            return false;
        if (component instanceof ZoneUIComponent && AAUtils.getZonesToRefresh(requestMap).contains(component.getId()))
            return false;
        else {
            for (Iterator iterator = component.getChildren().iterator(); iterator.hasNext();) {
                UIComponent child = (UIComponent) iterator.next();
                if (!canSkipRendering(child, context))
                    return false;
            }
            return true;
        }
    }
}
